<?php

namespace WKO\Controllers\Gateways;

use DateInterval;
use DateTime;
use WC_Payment_Gateway;
use WKO\Controllers\Koin;
use WKO\Controllers\Logs;
use WKO\Controllers\Render\BilletCheckout;
use WKO\Controllers\Render\BilletThankyou;
use WKO\Controllers\Webhooks;
use WKO\Helpers\Config;
use WKO\Model\Options;
use WKO\Model\PostMeta;

/**
 * Name: Billet
 * Structure the billet payment method
 * @package Controllers\Gateways
 * @since 1.0.0
 */
class Billet extends WC_Payment_Gateway implements InterfaceGateways
{

    public function __construct() {
        
        $this->id                 = "wc-koin-billet";
        $this->icon               = esc_url( Config::__image( 'koin/b-koin.png' ) );
        $this->has_fields         = false;
        $this->method_title       = __( "Koin - Payment in installments", 'wc-koin-official' );
        $this->method_description = sprintf( "<div class='description'>%s <a target='_blank' href='%s'>%s<i class='fa-solid fa-up-right-from-square'></i></a></div>",
                                        __( "Pay in installments by ticket slip at Koin!", 'wc-koin-official' ),
                                        esc_url( "https://www.koin.com.br/#contato" ),
                                        __( "Create your koin account.", 'wc-koin-official' ),
                                    );

        $this->supports = [
            "products"
        ];

        $this->init_form_fields();

        $this->init_settings();

        $this->title       = $this->get_option( "title" );
        $this->description = $this->get_option( "description" );
        $this->enabled     = $this->get_option( "enabled" );
        $this->testmode    = "yes" === $this->get_option( "testmode" );
        $this->secret_key  = $this->testmode ? $this->get_option( "test_secret_key" ) : $this->get_option( "secret_key" );

        add_action( 'woocommerce_thankyou_' . $this->id, [ $this, 'handle_koin_thankyou_page' ]);

        if ( is_admin() ) {
            add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, [ $this, 'process_admin_options' ] );
        }
        $this->get_webhook_url();
        new Webhooks( $this->id, get_class( $this ) );
    }

    /**
     * Create/Edit billet gateway options
     * @since 1.0.0
     * @return void
     */
    public function init_form_fields()
    {

        $this->form_fields = [
            "enabled" => [
                "title"       => __( "Enable", 'wc-koin-official' ),
                "label"       => __( "Enable payment in installments by ticket", 'wc-koin-official' ),
                "type"        => "checkbox",
                "description" => __( "Check this option to activate the Koin payment method", 'wc-koin-official' ),
                "default"     => "no",
                "desc_tip"    => true
            ],

            "title" => [
                "title"       => __( "Title", 'wc-koin-official' ),
                "type"        => "text",
                "description" => __( "This controls the title which the user sees during checkout.", 'wc-koin-official' ),
                "default"     => __( "Ticket payments by installments", 'wc-koin-official' ),
                "desc_tip"    => true
            ],

            "description" => [
                "title"       => __( "Description", 'wc-koin-official' ),
                "type"        => "textarea",
                "description" => __( "This controls the description which the user sees during checkout.", 'wc-koin-official' ),
                "default"     => __( "Koin - Ticket Slip", 'wc-koin-official' ),
                "desc_tip"    => true
            ],

            "store_iss" => [
                "title"       => __( "Store ISS", 'wc-koin-official' ),
                "type"        => "text",
                "description" => __( "Enter your account's store ISS.", 'wc-koin-official' )
            ],

            "store_name" => [
                "title"       => __( "Store code", 'wc-koin-official' ),
                "type"        => "text",
                "description" => __( "Enter your account's store code.", 'wc-koin-official' )
            ],

            "consumer_key" => [
                "title"       => __( "Consumer Key", 'wc-koin-official' ),
                "type"        => "text",
                "description" => __( "Enter your account's consumer key.", 'wc-koin-official' )
            ],

            "secret_key" => [
                "title"       => __( "Secret Key", 'wc-koin-official' ),
                "type"        => "password",
                "description" => __( "Enter your account's secret key.", 'wc-koin-official' )
            ],

            "testmode" => [
                "title"       => __( "Test mode", 'wc-koin-official' ),
                "label"       => __( "Enable Test Mode", 'wc-koin-official' ),
                "type"        => "checkbox",
                "description" => __( "Place the payment gateway in test mode using test API keys.", 'wc-koin-official' ),
                "default"     => "no",
                "desc_tip"    => true
            ],

            "test_store_iss" => [
                "title"       => __( "Store ISS", 'wc-koin-official' ),
                "type"        => "text",
                "description" => __( "Enter your test account's store ISS.", 'wc-koin-official' )
            ],

            "test_store_name" => [
                "title"       => __( "Test store code", 'wc-koin-official' ),
                "type"        => "text",
                "description" => __( "Enter your test account's store code.", 'wc-koin-official' )
            ],

            "test_consumer_key" => [
                "title"       => __( "Test Consumer Key", 'wc-koin-official' ),
                "type"        => "text",
                "description" => __( "Enter your account's test secret key", 'wc-koin-official' )
            ],

            "test_secret_key" => [
                "title"       => __( "Test Secret Key", 'wc-koin-official' ),
                "type"        => "password",
                "description" => __( "Enter your account's test consumer key", 'wc-koin-official' )
            ],

            "success_order_status" => [
                'title'       => __( 'Success status for orders', 'wc-koin-official' ),
                'type'        => 'select',
                'options'     => [ 
                    'wc-processing' => __( 'Processing', 'wc-koin-official' ),
                    'wc-completed'  => __( 'Completed', 'wc-koin-official' )
                ],
                'description' => __( 'This field defines what status the order must have after being successfully completed.', 'wc-koin-official' ),
            ],

            "enable_log" => [
                "title"       => "Logs",
                "label"       => __( "Enable Woocommerce Logs for Koin.", 'wc-koin-official' ),
                "type"        => "checkbox",
                "description" => sprintf(
                    "%s<a href='admin.php?page=wc-status&tab=logs'>Woocommerce->Status->Logs</a>",
                    __( "To View the logs click the link: ", 'wc-koin-official' ),
                )
            ],

            "enable_my_account" => [
                "title"       => __( "My Account", 'wc-koin-official' ),
                "label"       => __( "Enable Koin menu on my account page.", 'wc-koin-official' ),
                "type"        => "checkbox",
                "description" => __( "Create a menus on Woocommerce my account page.", 'wc-koin-official' ),
                "default"     => "yes"
            ],

            "enable_sync_option" => [
                "title"       => __( "Sync orders option", 'wc-koin-official' ),
                "label"       => __( "Enable manual order synchronization option.", 'wc-koin-official' ),
                "type"        => "checkbox",
                "description" => sprintf(
                    "<span class='warning'><i class='fas fa-exclamation-triangle'></i><strong>%s</strong></span>%s",
                    __( " Waring! ", 'wc-koin-official' ),
                    __( "Activating this option you will have access to a manual status synchronization option within the order pages. Only enable this option if you know what you are doing." , 'wc-koin-official' )
                ),
                "default"     => "no"
            ],

            "koin_logo" => [
                "type"        => "hidden",
                "default"     => esc_url( Config::__image( "koin/b-koin.png" ) ),
            ],
        ];
        
    }

    /**
     * Render the payment fields
     * @since 1.0.0
     * @return void
     */
    public function payment_fields()
    {

        if ( $this->description ) {

            if ( $this->testmode ) {

                $this->description .= __( " Test mode activate! In this mode transactions are not real.", 'wc-koin-official' );
                $this->description  = trim( $this->description );
            }
            
            echo wpautop( wp_kses_post( $this->description ) );
        }

        new BilletCheckout;
    }

    /**
     * Validate the payment fields
     * @since 1.0.0
     * @return boolean
     */
    public function validate_fields()
    {

        if( ! isset( $_POST[ "billing_persontype" ] ) || empty( $_POST[ "billing_persontype" ] ) ) {
            wc_add_notice(  'Person type field is required!', 'error' );
            return false;
        } 

        return true;
    }

    /**
     * Handle gateway process payment
     * @since 1.0.0
     * @param int $wc_order_id
     * @return void
     */
    public function process_payment( $wc_order_id )
    {
        global $woocommerce;
               
        $wc_order = wc_get_order( $wc_order_id );
        $body = $this->build_body( $wc_order );

        $koin = new Koin;
        $logs = new Logs;

        $response = $koin->create_order( $body );

        if ( isset( $response['body'] ) ) {
            $body = json_decode( $response['body'] );

            if ( empty( $body ) || isset( $body->message ) ) {
                $logs->create_order_error( __( "Koin error: ", 'wc-koin-official' ), $body->message );
                wc_add_notice(  __( "Koin: An unknown error has occurred. Please, contact Us.", 'wc-koin-official' ), "error" );

            } else {

                if ( isset( $body->status ) ) {
                    
                    $status = $body->status;

                    if ( array_intersect( [ $status->type ], [ 'Published', 'Opened' ] ) ) {

                        /** Changes order status and reducing stock */
                        $wc_order->update_status( 'wc-awaiting-analysis', sprintf( "<strong>%s</strong> :", __( WKO_PLUGIN_NAME, 'wc-koin-official' ) ), true );
                        wc_reduce_stock_levels( $wc_order_id );

                        /** Confirmation waiting warning message */
                        $wc_order->add_order_note( sprintf( "<strong>%s</strong> : %s", 
                            __( WKO_PLUGIN_NAME, 'wc-koin-official' ), 
                            __( "Waiting for credit confirmation from Koin.", 'wc-koin-official' )
                        ), true );

                        $payment_link = isset( $body->return_url ) ? $body->return_url : '';
                        /** Inform the koin payment link */
                        $wc_order->add_order_note( sprintf( "<strong>%s</strong> : %s %s",
                            __( WKO_PLUGIN_NAME, 'wc-koin-official' ), 
                            __( "Koin payment link:", 'wc-koin-official' ),
                            $payment_link
                        ), true );


                        if ( $this->testmode === 'yes' ) {
                            $wc_order->add_order_note( sprintf( "<strong>%s</strong> : %s", 
                                __( WKO_PLUGIN_NAME, 'wc-koin-official' ), 
                                __( "Test mode activate! In this mode transactions are not real.", 'wc-koin-official' )
                            ), true );
                        }

                        $metas = [
                            '_payment_link' => $payment_link
                        ];  

                        /** Save order post metas */
                        $this->save_post_meta( $wc_order_id, $metas );

                        /** Empty cart */
                        $woocommerce->cart->empty_cart();

                        /** Send success woocommerce log */
                        $logs->create_order_success( __( "Koin Success Order: ", 'wc-koin-official' ), json_encode( $response ) );

                        /** Redirect user to the thank you page */
                        return array(
                            'result' => 'success',
                            'redirect' => $this->get_return_url( $wc_order )
                        );
                    } else {

                        $logs->create_order_error( __( "Koin error - Invalid order status: ", 'wc-koin-official' ), json_encode( $response ) );
                        wc_add_notice(  __( "An unknown error has occurred. Please, contact Us.", 'wc-koin-official' ), "error" );
                    }

                }
                
            }
        } else {
            $logs->create_order_error( __( "No body found: ", 'wc-koin-official' ),json_encode( $response ) );
            wc_add_notice(  __( "Koin: An unknown error has occurred. Please, contact Us.", 'wc-koin-official' ), "error" );
        }
    }

    /**
     * Make the body request 
     * @since 1.0.0
     * @param object $wc_order
     * @return array
     */
    private function build_body( $wc_order )
    {
        $address  = $this->get_address_data( $wc_order );
        $user     = $this->get_user_data( $wc_order );
        $person   = $this->get_person_info( $wc_order->get_meta_data() );

        $delivery_date = new DateTime();
        $delivery_date->add( new DateInterval('P30D') );

        $expiration_date = new DateTime();
        $expiration_date->add( new DateInterval('P30D') );

        $ip = "";

        if ( isset( $_SERVER['HTTP_CLIENT_IP'] ) && $_SERVER['HTTP_CLIENT_IP'] ) {
            $ip = sanitize_text_field( $_SERVER['HTTP_CLIENT_IP'] );

        } else {

            if ( isset( $_SERVER['HTTP_X_FORWARDED_FOR'] ) && $_SERVER['HTTP_X_FORWARDED_FOR'] ) {
                $ip = sanitize_text_field( $_SERVER['HTTP_X_FORWARDED_FOR'] );

            } else if ( isset( $_SERVER['REMOTE_ADDR'] ) && $_SERVER['REMOTE_ADDR'] ){
                $ip = sanitize_text_field( $_SERVER['REMOTE_ADDR'] );
            }
        }

        $device_fingerprint = array_map( 'sanitize_text_field', $_SERVER );
        $device_fingerprint = md5( serialize( $device_fingerprint ) );

        $store_name = $this->get_option("store_name");

        if ( $this->get_option("testmode") === 'yes' ) {
            $store_name = $this->get_option("test_store_name");
        }

        $domain = sanitize_text_field( $_SERVER['HTTP_HOST'] );

        $bytes = random_bytes(6);
        $token = bin2hex( $bytes );

        $body = [
            "store" => [
                'category' => "",
                'code'     => $store_name
            ],
            "transaction" => [
                "amount" => [
                    "value"     => $wc_order->get_total(),
                    "breakdown" => [
                        "items" => [
                            "currency_code" => $wc_order->get_currency(),
                            "value"         => $wc_order->get_total()
                        ],
                        "shipping" => [
                            "currency_code" => $wc_order->get_currency(),
                            "value"         => $wc_order->get_shipping_total()
                        ],
                        "taxes" => [
                            "currency_code" => $wc_order->get_currency(),
                            "value"         => $wc_order->get_total_tax()
                        ]
                    ],
                    "currency_code" => $wc_order->get_currency()
                ],
                "reference_id" => $token ."_". $domain . "_" . $wc_order->get_id()
            ],
            "payment_method" => [
                "code" => "BNPL",
                "expiration_date" => str_replace( "+00:00", "Z", $expiration_date->format('c') )
            ],
            "payer" => [
                "phone" => [
                     "country_code" => 55,
                     "number" => $user['phone'],
                     "area"   => $user['area'],
                     "type"   => "phone"
                ],
                "address" => [
                     "country_code" => $address['billing']['country'],
                     "state"        => $address['billing']['state'],
                     "city_name"    => $address['billing']['city'],
                     "street"       => $address['billing']['address_1'],
                     "number"       => $address['billing']['number'],
                     "zip_code"     => $address['billing']['postcode'],
                     "district"     => $address['billing']['neighborhood']
                ],
                "first_name" => $user['first_name'],
                "last_name"  => $user['last_name'],
                "full_name"  => $user['full_name'],
                "email"      => $user['email'],
                "document" => [
                    "type"        => $person['type'],
                    "number"      => $person['id'],
                    "nationality" => $address['billing']['country']
                ]
            ],
            "shipping" => [
                "address" => [
                    "country_code" => $address['shipping']['country'],
                    "state"        => $address['shipping']['state'],
                    "city_name"    => $address['shipping']['city'],
                    "street"       => $address['shipping']['address_1'],
                    "number"       => $address['shipping']['number'],
                    "zip_code"     => $address['shipping']['postcode'],
                    "district"     => $address['shipping']['neighborhood']
                ],
                "delivery_date" => $delivery_date->format('Y-m-d')
            ],
            "items" => $this->get_order_itens( $wc_order ),
            "notification_url" => [
                $this->get_webhook_url()
            ],
            "payment_url"        => get_permalink( wc_get_page_id( 'myaccount' ) ) . "view-order/{$wc_order->get_id()}" ,
            "verified_id"        => false,
            "ip_address"         => $ip,
            "country_code"       => $address['billing']['country'],
            "device_fingerprint" => $device_fingerprint
        ];

        return $body;
    }


    /**
     * Get return URL
     * @since 1.0.0
     * @param int $wc_order_id
     * @return array
     */
    private function get_address_data( $wc_order )
    {
        $billing  = $wc_order->get_address("billing");
        $shipping = $wc_order->get_address("shipping");

        if ( ! $shipping['country'] ) {
            $shipping = $billing;
        }

        return [
            'billing'  => $billing,
            'shipping' => $shipping
        ];
        
    }

    /**
     * Get user data
     * @since 1.0.0
     * @param object $wc_order
     * @return array
     */
    private function get_user_data( $wc_order )
    {
        $billing_first_name = $wc_order->get_billing_first_name();
        $billing_last_name  = $wc_order->get_billing_last_name();
        $billing_email      = $wc_order->get_billing_email();
        $billing_phone      = $wc_order->get_billing_phone();

        $first_name = $billing_first_name ? $billing_first_name : "";
        $last_name  = $billing_last_name ? $billing_last_name : "";
        $email      = $billing_email ? $billing_email : "";

        $phone      = $billing_phone ? str_replace( [ ' ', '(', ')', '-' ], '', $billing_phone ) : "";
        $area       = $phone;
        
        $area       = $area ? preg_replace('/\A.{2}?\K[\d]+/', '', $area) : "";
        $phone      = $phone ? preg_replace('/^\d{2}/', '', $phone)  : "";

        $full_name  = "$first_name $last_name";

        return [
            "first_name" => $first_name,
            "last_name"  => $last_name,
            "full_name"  => $full_name,
            "email"      => $email,
            'phone'      => $phone,
            'area'       => $area
        ];
    }

    /**
     * Get order itens
     * @since 1.0.0
     * @param int $itens
     * @return array
     */
    private function get_order_itens( $wc_order )
    {
        $_items = [];

        foreach ( $wc_order->get_items() as $item ) {

            $data = $item->get_data();
            $cats = wc_get_product_term_ids( $data['product_id'], 'product_cat' );

            if ( empty( $cats ) ) {
                $cat = [ 'name' => 'Uncategorized' ];

            } else {
                $cat = get_term_by( 'id', $cats[0], 'product_cat' );

            }

            $_item = [
                
                "price" => [
                    "currency_code" => $wc_order->get_currency(),
                    "value" => $data['total']
                ],
                    "reference" => $data['product_id'],
                    "name"      => $data['name'],
                    "quantity"  => $data['quantity'],
                    "discount"  => 0,
                    "category"  => $cat->name
                ];

            array_push( $_items, $_item );
        }
         
        return $_items;
    }

    /**
     * Get person type info
     * @since 1.0.0
     * @param array $wc_meta_data
     * @return array
     */
    private function get_person_info( $wc_meta_data )
    {
        $person_type  = 1;
        $billing_cnpj = "";
        $billing_cpf  = "";

        foreach( $wc_meta_data as $meta ) {
            $data = $meta->get_data();

            if ( $data['key'] === '_billing_persontype' ) {
                $person_type = $data['value'];
            }

            if ( $data['key'] === '_billing_cnpj' ) {
                $billing_cnpj = $data['value'];
            }

            if ( $data['key'] === '_billing_cpf' ) {
                $billing_cpf = $data['value'];
            }
        }

        $person = [];

        if ( $person_type == 1 ) {
            $person['type'] = "cpf";
            $person['id']   = str_replace( [ '.','-' ], '', $billing_cpf );
        } else {
            $person['type'] = "cnpj";
            $person['id']   = str_replace( [ '.', '/', '-', ], '', $billing_cnpj);
        }

        return $person;
    }

    /**
     * Save order meta values
     * @param int $order
     * @param array $metas
     * @return void
     */
    private function save_post_meta( $order, $metas )
    {
        $postmeta = new PostMeta;
        
        foreach( $metas as $key => $meta ) {
            $postmeta->create( $order, $key, $meta );
        }
    }

    /**
     * Get Webhook URL
     * @since 1.0.0
     * @return string
     */
    private function get_webhook_url()
    {
        $opt   = new Options;
        $token = $opt->get( "_webhook_token_$this->id" );
        $url   = get_option( 'siteurl' );
        
        return "$url/wc-api/$token-$this->id";
    }

    /**
     * Call thankyou page render
     * @since 1.0.0
     * @return object
     */
    public function handle_koin_thankyou_page()
    {
        return new BilletThankyou;
    }
}