<?php

namespace WKO\Controllers\Webhooks;

use WKO\Controllers\Logs;
use WKO\Model\Options;
use WKO\Model\PostMeta;

/**
 * Name: Billet
 * Handle Billet Webhook
 * @package Controllers\Webhooks
 * @since 1.0.0
 */
class Billet implements InterfaceWebhooks
{
    private $logs;
    private $postmeta;
    private $metas;

    public function __construct()
    {
        $this->logs     = new Logs;
        $this->postmeta = new PostMeta;
    }
    /**
     * Receive notifications
     * @since 1.0.0
     * @return void
     */
    public function handle_notifications()
    {
        $post_data = file_get_contents( 'php://input' );

		$body = empty( $post_data ) ? false : json_decode( $post_data );
        $metas = [];

        if ( $body && is_object( $body ) ) {

            if ( isset( $body->status ) ) {

                $status = $body->status;
                $order_status = isset( $status->type ) ? $status->type : false;

                $statuses = [
                    'Collected',
                    'Cancelled',
                    'Pending',
                    'Waiting',
                    'Failed'
                ];
                
                if ( array_intersect( $statuses, [ $order_status ] ) ) {

                    $transaction  = $body->transaction;
                    $installments = $body->installment_option;
                    $reference_id = isset( $transaction->reference_id ) ? $transaction->reference_id : false;

                    $reference_id = explode( "_", $reference_id );
                    
                    if ( ! empty( $reference_id ) && is_array( $reference_id ) ) {
                        $reference_id = $reference_id[ count($reference_id) - 1 ];
                    } else {
                        $reference_id = false;
                    }

                    $this->wc_order_id            = intval( $reference_id ) ? $reference_id : false;
                    $this->metas['_order_id']     = isset( $body->order_id ) ? $body->order_id : false;
                    $this->metas['_installments'] = isset( $installments->installments ) ? $installments->installments : false;

                    if (  $this->validate_metas( $metas, $body ) ) {

                        $this->handle_statuses( $order_status );

                        $title = __( "KOIN WEBHOOK SUCCESS", 'wc-koin-official' );
    
                        $message = sprintf( "%s %s", 
                            __( "Notification received successfully!", 'wc-koin-official' ),
                            __( "\nBody request:\n" . json_encode( $body ) . "\n\n", 'wc-koin-official' )
                        );
                        
                        $this->logs->webhook_notice_success( $title, $message );
                    }

                } else {
                    $title = __( "KOIN WEBHOOK ERROR", 'wc-koin-official' );
    
                    $message = sprintf( "%s %s", 
                        __( "Notification status invalid!", 'wc-koin-official' ),
                        __( "\nBody request:\n" . json_encode( $body ) . "\n\n", 'wc-koin-official' )
                    );
                    
                    $this->logs->webhook_notice_error( $title, $message );
                }
            }
        }
    }

    private function handle_statuses( $status )
    {
        $opt = new Options;

        $order_status = false;
        $success_status = $opt->get_gateways_option( 'success_order_status' );

        if ( ! $success_status ) {
            $success_status = "wc-processing";
        } 

        switch ( $status ) {
            case 'Waiting':
                $order_status = 'wc-awaiting-payment';
                $message = __( "Waiting for first payment.", 'wc-koin-official' );
                break;
            case 'Pending':
                $order_status = 'wc-awaiting-analysis';
                $message = __( "Awaiting credit analysis by Koin.", 'wc-koin-official' );
                break;
            case 'Collected':
                $order_status = $success_status;
                $message = __( "Order completed successfully!", 'wc-koin-official' );
                break;
            case 'Cancelled':
                $order_status = 'wc-cancelled';
                $message = __( "Order cancelled successfully!", 'wc-koin-official' );
                break;
            case 'Failed':
                $order_status = 'wc-failed';
                $message = __( "Order failed!", 'wc-koin-official' );
                break;
        }


        if ( $order_status ) {
            $wc_order = wc_get_order( $this->wc_order_id );
            $wc_order->update_status( $order_status, sprintf( "<strong>%s</strong> : %s -  %s", 
                    __( WKO_PLUGIN_NAME, 'wc-koin-official' ),
                    __( "Status changed automatically by Koin", 'wc-koin-official' ),
                    $message
                ), 
                true 
            );
        }
    }

    
    private function validate_metas( $_body )
    {
        $failed = false;

        $metas = [
            "_order_id",
            "_installments",
        ];

        foreach ( $metas as $meta ) {

            if ( ! isset( $this->metas[ $meta ] ) ) {

                $title = __( "KOIN WEBHOOK ERROR", 'wc-koin-official' );

                $message = sprintf( "%s %s %s %s", 
                    __( "It was not possible to identify the ", 'wc-koin-official' ),
                    $meta,
                    __( " field in the received notification.", 'wc-koin-official' ),
                    __( "\nBody request:\n" . json_encode( $_body ) . "\n\n", 'wc-koin-official' )
                );

                $failed = true;

                $this->logs->webhook_notice_error( $title, $message );
            }
        }

        if ( ! $failed ) {
            $this->save_metas( $_body );
            return true;
        }

        return false;
    }
	
    private function save_metas( $_body )
    {
        $metas = [
            "_order_id",
            "_installments",
        ];

        $order = wc_get_order( $this->wc_order_id );

        if ( $order ) {
            foreach ( $this->metas as $key => $meta ) {
                if ( array_intersect( $metas, [ $key ] ) ) {
                    $this->postmeta->update( $this->wc_order_id, $key, $meta );
                }
            }

        } else {

            $title = __( "KOIN WEBHOOK ERROR", 'wc-koin-official' );

            $message = sprintf( "%s %s %s", 
                __( "Woocomerce orde not found!", 'wc-koin-official' ),
                __( "\nOrder ID: $this->wc_order_id", 'wc-koin-official' ),
                __( "\nBody request:\n" . json_encode( $_body ) . "\n\n", 'wc-koin-official' )
            );

            $this->logs->webhook_notice_error( $title, $message );
        }
    }
}